=== modified file '.bzrignore'
--- .bzrignore	2011-01-22 16:51:51 +0000
+++ .bzrignore	2011-02-11 23:56:37 +0000
@@ -315,6 +315,7 @@
 tests/Makefile.in
 tests/dtr
 tests/dtr@
+tests/dbqp_data/uuid
 tests/dbqp_work
 tests/install_test_db
 tests/mtr

=== modified file 'docs/testing/dbqp.rst'
--- docs/testing/dbqp.rst	2011-01-30 23:03:45 +0000
+++ docs/testing/dbqp.rst	2011-02-12 00:01:28 +0000
@@ -314,13 +314,21 @@
    Turn off the use of --secure-file-priv=vardir for
    started servers
 
+Options to pass options on to the server
+-----------------------------------------
+.. option:: --drizzled=DRIZZLEDOPTIONS
+           
+    Pass additional options to the server.  Will be passed
+    to all servers for all tests (mostly for --start-and-
+    exit)
+
+
 Options for defining the tools we use for code analysis (valgrind, gprof, gcov, etc)
 ---------------------------------------------------------------------------------------
 .. option:: --valgrind          
 
    Run drizzletest and drizzled executables using
    valgrind with default options [False]
-   (CURRENTLY INOP)
 
 .. option:: --valgrind-option=VALGRINDARGLIST
                        
@@ -332,8 +340,13 @@
 .. option:: --gdb
 
     Start the drizzled server(s) in gdb
-    (CURRENTLY INOP)
-
+
+.. option:: --manual-gdb
+
+    Allows you to start the drizzled server(s) in gdb
+    manually (in another window, etc
+
+    
 
 
 

=== modified file 'tests/include.am'
--- tests/include.am	2011-02-10 23:54:22 +0000
+++ tests/include.am	2011-02-11 23:56:37 +0000
@@ -147,7 +147,7 @@
 test-plugins: 
 	( $(MAKE) $(AM_MAKEFLAGS) && cd tests && $(TEST_RUN) $(TEST_RUN_EXTRA_OPTIONS) --fast --force --suite=$(PLUGIN_TESTS) )
 
-test-dbqp: check
+test-dbqp: 
 	( $(MAKE) $(AM_MAKEFLAGS) && cd tests && $(TEST_RUN_DBQP) $(TEST_RUN_EXTRA_OPTIONS) --fast --force --suite=$(NORMAL_TESTS) )
 
 

=== modified file 'tests/lib/server_mgmt/drizzled.py'
--- tests/lib/server_mgmt/drizzled.py	2011-02-04 20:41:08 +0000
+++ tests/lib/server_mgmt/drizzled.py	2011-02-11 23:56:37 +0000
@@ -61,6 +61,7 @@
 
         self.system_manager = self.server_manager.system_manager
         self.valgrind = self.system_manager.valgrind
+        self.gdb = self.system_manager.gdb
         if self.valgrind:
             self.valgrind_time_buffer = 10
         else:
@@ -157,20 +158,31 @@
             as desired / intended
  
         """
-      
-        return "%s %s %s --mysql-protocol.port=%d --mysql-protocol.connect-timeout=60 --mysql-unix-socket-protocol.path=%s --pid-file=%s --drizzle-protocol.port=%d --datadir=%s --tmpdir=%s --innodb.data-file-path=ibdata1:20M:autoextend --sort-buffer-size=256K --max-heap-table-size=1M %s %s > %s 2>&1 & " % ( self.cmd_prefix
-                                               , self.server_path
-                                               , self.process_server_options()
-                                               , self.master_port
-                                               , self.socket_file
-                                               , self.pid_file
-                                               , self.drizzle_tcp_port
-                                               , self.datadir
-                                               , self.tmpdir
-                                               , self.secure_file_string
-                                               , self.user_string 
-                                               , self.error_log
-                                               )
+
+        server_args = [ self.process_server_options()
+                      , "--mysql-protocol.port=%d" %(self.master_port)
+                      , "--mysql-protocol.connect-timeout=60"
+                      , "--innodb.data-file-path=ibdata1:20M:autoextend"
+                      , "--sort-buffer-size=256K"
+                      , "--max-heap-table-size=1M"
+                      , "--mysql-unix-socket-protocol.path=%s" %(self.socket_file)
+                      , "--pid-file=%s" %(self.pid_file)
+                      , "--drizzle-protocol.port=%d" %(self.drizzle_tcp_port)
+                      , "--datadir=%s" %(self.datadir)
+                      , "--tmpdir=%s" %(self.tmpdir)
+                      , self.secure_file_string
+                      , self.user_string
+                      ]
+
+        if self.gdb:
+            server_args.append('--gdb')
+            return self.system_manager.handle_gdb_reqs(self, server_args)
+        else:
+            return "%s %s %s & " % ( self.cmd_prefix
+                                   , self.server_path
+                                   , " ".join(server_args)
+                                   )
+
 
     def get_stop_cmd(self):
         """ Return the command that will shut us down """

=== modified file 'tests/lib/server_mgmt/server_management.py'
--- tests/lib/server_mgmt/server_management.py	2011-02-04 20:41:08 +0000
+++ tests/lib/server_mgmt/server_management.py	2011-02-11 23:56:37 +0000
@@ -39,6 +39,9 @@
     def __init__(self, system_manager, variables):
         self.skip_keys = [ 'ld_lib_paths'
                          , 'system_manager'
+                         , 'logging'
+                         , 'gdb'
+                         , 'code_tree'
                          ]
         self.debug = variables['debug']
         self.verbose = variables['verbose']
@@ -47,12 +50,14 @@
         self.no_secure_file_priv = variables['nosecurefilepriv']
         self.system_manager = system_manager
         self.logging = system_manager.logging
+        self.gdb  = self.system_manager.gdb
         self.code_tree = system_manager.code_tree
+        self.user_server_opts = variables['drizzledoptions']
         self.servers = {}
         # We track this
         self.ld_lib_paths = system_manager.ld_lib_paths
         self.mutex = thread.allocate_lock()
-        self.timer_increment = .25
+        self.timer_increment = .5
 
         if self.debug:
             self.logging.debug_class(self)
@@ -92,10 +97,11 @@
             Start up occurs elsewhere
 
         """
+
         # Get a name for our server
         server_name = self.get_server_name(requester)
 
-        # initialize a server_object
+        # initialize our new server_object
         if self.code_tree.type == 'Drizzle':
           from lib.server_mgmt.drizzled import drizzleServer as server_type
         new_server = server_type( server_name, self, server_options
@@ -139,15 +145,39 @@
         # we don't know the server is running (still starting up)
         # so we give it a few minutes
         self.tried_start = 1
-        server_subproc = subprocess.Popen( start_cmd
-                                         , shell=True
-                                         , env=working_environ
-                                         )
-        server_subproc.wait()
-        server_retcode = server_subproc.returncode
-        #print server_subproc.pid
+        error_log = open(server.error_log,'w')
+        if start_cmd: # It will be none if --manual-gdb used
+            if not self.gdb:
+                server_subproc = subprocess.Popen( start_cmd
+                                                 , shell=True
+                                                 , env=working_environ
+                                                 , stdout=error_log
+                                                 , stderr=error_log
+                                                 )
+                server_subproc.wait()
+                server_retcode = server_subproc.returncode
+            else: 
+                # This is a bit hackish - need to see if there is a cleaner
+                # way of handling this
+                # It is annoying that we have to say stdout + stderr = None
+                # We might need to further manipulate things so that we 
+                # have a log
+                server_subproc = subprocess.Popen( start_cmd
+                                                 , shell=True
+                                                 , env = working_environ
+                                                 , stdin=None
+                                                 , stdout=None
+                                                 , stderr=None
+                                                 , close_fds=True
+                                                 )
+        
+                server_retcode = 0
+        else:
+            # manual-gdb issue
+            server_retcode = 0
         timer = float(0)
         timeout = float(server.server_start_timeout)
+
         #if server_retcode: # We know we have an error, no need to wait
         #    timer = timeout
 
@@ -157,8 +187,11 @@
         while not self.system_manager.find_path( [server.pid_file]
                                                , required=0) and timer != timeout:
             time.sleep(self.timer_increment)
-            timer= timer + self.timer_increment
-
+            # If manual-gdb, this == None and we want to give the 
+            # user all the time they need
+            if start_cmd:
+                timer= timer + self.timer_increment
+            
         if timer == timeout and not self.ping_server(server, quiet=True):
             self.logging.error(( "Server failed to start within %d seconds.  This could be a problem with the test machine or the server itself" %(timeout)))
             server_retcode = 1
@@ -275,6 +308,8 @@
         current_servers = self.servers[requester]
         
         for server in current_servers:
+            # We add in any user-supplied options here
+            server_options = server_options + self.user_server_opts
             if self.compare_options( server.server_options
                                    , server_options):
                 return 1

=== modified file 'tests/lib/sys_mgmt/system_management.py'
--- tests/lib/sys_mgmt/system_management.py	2011-02-09 17:41:08 +0000
+++ tests/lib/sys_mgmt/system_management.py	2011-02-11 23:56:37 +0000
@@ -75,6 +75,7 @@
         self.start_dirty = variables['startdirty']
         self.valgrind = variables['valgrind']
         self.gdb = variables['gdb']
+        self.manual_gdb = variables['manualgdb']
 
         # we use this to preface commands in order to run valgrind and such
         self.cmd_prefix = '' 
@@ -130,10 +131,10 @@
         # check for libtool
         self.libtool = self.libtool_check()
 
-        # do we need to setup for valgrind?
-        if self.valgrind:
-            self.handle_valgrind_reqs(variables['valgrindarglist'])
-            
+        # See if we need to do any further processing for special
+        # options like valgrind and gdb
+        self.handle_additional_reqs(variables)
+     
         if self.debug:
             self.logging.debug_class(self)
         
@@ -213,8 +214,6 @@
             uuid_file.close()
         else:
             uuid = uuid4()
-            print uuid
-            print uuid4()
             uuid_file = open(uuid_file_name,'w')
             uuid_file.write(str(uuid))
             uuid_file.close()
@@ -436,11 +435,87 @@
         else:
             return None
 
+    def handle_additional_reqs(self, variables):
+        """ Do what we need to do to set things up for
+            options like valgrind and gdb
+
+        """
+
+        # do we need to setup for valgrind?
+        if self.valgrind:
+            self.handle_valgrind_reqs(variables['valgrindarglist'])
+
+    def handle_gdb_reqs(self, server, server_args):
+        """ We generate the gdb init file and whatnot so we
+            can run gdb properly
+
+            if the user has specified manual-gdb, we provide
+            them with a message about when to start and
+            signal the server manager to simply wait
+            for the server to be started by the user
+
+        """
+        extra_args = ''
+        gdb_term_cmd = "xterm -title %s.%s " %( server.owner
+                                              , server.name
+                                              )
+        gdb_file_name = "%s.gdbinit" %(server.name)
+
+        if self.cur_os == 'Darwin': # Mac...ick ; P
+            extra_args = [ "set env DYLD_INSERT_LIBRARIES /usr/lib/libgmalloc.dylib"
+                         , "set env MallocStackLogging 1"
+                         , "set env MallocScribble 1"
+                         , "set env MallocPreScribble 1"
+                         , "set env MallocStackLogging 1"
+                         , "set env MallocStackLoggingNoCompact 1"
+                         , "set env MallocGuardEdges 1"
+                         ] 
+
+        # produce our init file
+        if extra_args:
+            extra_args = "\n".join(extra_args)
+        gdb_file_contents = [ "set args %s" %(" ".join(server_args))
+                            , "%s" % (extra_args)
+                            , "set breakpoint pending on"
+	                          , "break drizzled::parse"
+	                          , "commands 1"
+                            , "disable 1"
+	                          , "end"
+                            , "set breakpoint pending off"
+	                          , "run"
+                            ]
+        gdb_file_path = os.path.join(server.tmpdir, gdb_file_name)
+        gdb_init_file = open(gdb_file_path,'w')
+        gdb_init_file.write("\n".join(gdb_file_contents))
+        gdb_init_file.close()
+
+        # return our command line
+        if self.libtool:
+            libtool_string = "%s --mode=execute " %(self.libtool)
+        else:
+            libtool_string = ""
+
+        if self.manual_gdb:
+            self.logging.info("To start gdb, open another terminal and enter:")
+            self.logging.info("%s/../libtool --mode=execute gdb -cd %s -x %s %s" %( self.code_tree.testdir
+                                                                                  , self.code_tree.testdir
+                                                                                  , gdb_file_path
+                                                                                  , server.server_path
+                                                                                  ) )
+            return None
+
+        else:
+            return "%s -e %s gdb -x %s %s" %( gdb_term_cmd
+                                            , libtool_string
+                                            , gdb_file_path
+                                            , server.server_path
+                                            )
+
     def handle_valgrind_reqs(self, optional_args, mode='valgrind'):
         """ We do what voodoo we need to do to run valgrind """
         valgrind_args = [ "--show-reachable=yes"
-                        , "--malloc-fill=55"
-                        , "--free-fill=55"
+                        , "--malloc-fill=22"
+                        , "--free-fill=22"
                         # , "--trace-children=yes" this is for callgrind only
                         ]
         if optional_args:

=== modified file 'tests/lib/test_mgmt/test_execution.py'
--- tests/lib/test_mgmt/test_execution.py	2011-02-07 22:20:03 +0000
+++ tests/lib/test_mgmt/test_execution.py	2011-02-11 23:56:37 +0000
@@ -83,9 +83,7 @@
         #self.test_manager.mutex.acquire()
         self.current_testcase = self.test_manager.get_testCase(self.name)
         #self.test_manager.mutex.release()
-        if self.debug:
-            self.logging.debug("Executor: %s, assigned test: %s" %(self.name
-                                            , self.current_testcase.fullname))
+        
 
     def handle_server_reqs(self, start_and_exit):
         """ Get the servers required to execute the testCase 

=== modified file 'tests/lib/test_run_options.py'
--- tests/lib/test_run_options.py	2011-02-04 20:41:08 +0000
+++ tests/lib/test_run_options.py	2011-02-11 23:56:37 +0000
@@ -51,6 +51,8 @@
     variables = {}
     variables = vars(args)
     variables['test_cases']= test_cases
+    if variables['manualgdb']:
+        variables['gdb']=True
     return variables
 
 # Create the CLI option parser
@@ -291,10 +293,25 @@
   , help = "Turn off the use of --secure-file-priv=vardir for started servers"
   )
 
- 
 parser.add_option_group(environment_control_group)
 # end environment control group
 
+option_passing_group = optparse.OptionGroup(parser,
+                          "Options to pass options on to the server")
+
+option_passing_group.add_option(
+    "--drizzled"
+  , dest="drizzledoptions"
+  , type='string'
+  , action='append' 
+  , default = []
+  , help = "Pass additional options to the server.  Will be passed to all servers for all tests (mostly for --start-and-exit)"
+  )
+
+parser.add_option_group(option_passing_group)
+# end option passing group
+ 
+
 analysis_control_group = optparse.OptionGroup(parser, 
                             "Options for defining the tools we use for code analysis (valgrind, gprof, gcov, etc)")
 
@@ -326,6 +343,14 @@
   , help="Start the drizzled server(s) in gdb"
   )
 
+debugger_control_group.add_option(
+    "--manual-gdb"
+  , dest="manualgdb"
+  , action='store_true'
+  , default=False
+  , help="Allows you to start the drizzled server(s) in gdb manually (in another window, etc)"
+  )
+
 parser.add_option_group(debugger_control_group)
 
 

